package org.example.wedding_app_backend.api.controller;

import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingImage.controller.WeddingImageController;
import org.example.wedding_app_backend.weddingImage.dto.GetWeddingImageResponse;
import org.example.wedding_app_backend.weddingImage.entity.WeddingImageEntity;
import org.example.wedding_app_backend.weddingImage.function.RequestToWeddingImageFunction;
import org.example.wedding_app_backend.weddingImage.function.WeddingImagesMapper;
import org.example.wedding_app_backend.weddingImage.service.WeddingImageService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.hamcrest.CoreMatchers.is;
import static org.mockito.BDDMockito.when;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;

@WebMvcTest(controllers = WeddingImageController.class)
@AutoConfigureMockMvc(addFilters = false)
public class WeddingImageControllerTests {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private WeddingImageService weddingImageService;

    @MockBean
    private RequestToWeddingImageFunction requestToWeddingImageFunction;

    @MockBean
    private WeddingImagesMapper weddingImagesMapper;

    private WeddingEntity weddingEntity;

    private WeddingGuestEntity weddingGuestEntity;

    private WeddingImageEntity weddingImageEntity;

    private GetWeddingImageResponse getWeddingImageResponse;

    @BeforeEach
    void setUp() {
        weddingEntity = WeddingEntity.builder().id(UUID.randomUUID()).build();
        weddingGuestEntity = WeddingGuestEntity.builder().id(UUID.randomUUID()).wedding(weddingEntity).build();
        weddingImageEntity = WeddingImageEntity.builder().id(UUID.randomUUID()).wedding(weddingEntity).build();
        getWeddingImageResponse = GetWeddingImageResponse.builder().id(weddingImageEntity.getId()).build();
    }

    @Test
    void getWeddingGallery_ShouldReturnImages_WhenImagesExist() throws Exception {
        when(weddingImageService.findAllWeddingImages(any(), any(), any()))
                .thenReturn(Optional.of(List.of(getWeddingImageResponse)));

        ResultActions response = mockMvc.perform(get("/api/wedding/{weddingId}/{guestId}/gallery", weddingEntity.getId(), weddingGuestEntity.getId())
                .param("sort", "date")
                .contentType(MediaType.APPLICATION_JSON));

        response.andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.jsonPath("$.size()", is(1)))
                .andDo(MockMvcResultHandlers.print());
    }

    @Test
    void getWeddingGallery_ShouldReturnNotFound_WhenNoImagesExist() throws Exception {
        when(weddingImageService.findAllWeddingImages(any(), any(), any())).thenReturn(Optional.empty());

        ResultActions response = mockMvc.perform(get("/api/wedding/{weddingId}/{guestId}/gallery", weddingEntity.getId(), weddingGuestEntity.getId())
                .param("sort", "date")
                .contentType(MediaType.APPLICATION_JSON));

        response.andExpect(MockMvcResultMatchers.status().isNotFound());
    }

    @Test
    void putWeddingImage_ShouldSaveImage() throws Exception {
        MockMultipartFile mockFile = new MockMultipartFile("image", "test.jpg", "image/jpeg", new byte[]{1, 2, 3});

        when(requestToWeddingImageFunction.apply(any(), any())).thenReturn(weddingImageEntity);
        doNothing().when(weddingImageService).saveWeddingImage(any());

        ResultActions response = mockMvc.perform(multipart("/api/wedding/{weddingId}/gallery", weddingEntity.getId())
                .file(mockFile)
                .contentType(MediaType.MULTIPART_FORM_DATA));

        response.andExpect(MockMvcResultMatchers.status().isCreated());
    }

    @Test
    void changeWeddingImageLikes_ShouldUpdateLikes_WhenImageExists() throws Exception {
        when(weddingImageService.findWeddingImage(any())).thenReturn(Optional.of(weddingImageEntity));

        doNothing().when(weddingImageService).updateLikes(any(), any(), anyBoolean());

        ResultActions response = mockMvc.perform(patch("/api/wedding/{weddingId}/{guestId}/gallery/{imageId}", weddingEntity.getId(), weddingGuestEntity.getId(), weddingImageEntity.getId())
                .param("liked", "true")
                .contentType(MediaType.APPLICATION_JSON));

        response.andExpect(MockMvcResultMatchers.status().isOk());
    }

    @Test
    void changeWeddingImageLikes_ShouldThrowNotFound_WhenImageDoesNotExist() throws Exception {
        when(weddingImageService.findWeddingImage(any())).thenReturn(Optional.empty());

        ResultActions response = mockMvc.perform(patch("/api/wedding/{weddingId}/{guestId}/gallery/{imageId}", weddingEntity.getId(), weddingGuestEntity.getId(), weddingImageEntity.getId())
                .param("liked", "true")
                .contentType(MediaType.APPLICATION_JSON));

        response.andExpect(MockMvcResultMatchers.status().isNotFound());
    }
}

