import {
  Component,
  HostListener, Input,
  OnInit,
  signal,
  WritableSignal
} from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { GalleryService } from '../../services/gallery.service';
import { NgClass, NgForOf, NgIf } from '@angular/common';
import { CascadeSelectModule } from 'primeng/cascadeselect';
import { PanelModule } from 'primeng/panel';
import {FileSelectEvent, FileUploadModule} from 'primeng/fileupload';
import { ButtonGroupModule } from 'primeng/buttongroup';
import { catchError } from 'rxjs';
import {ImageType} from '../../models/image.type';
import {MenuItem} from 'primeng/api';
import {TabMenuModule} from 'primeng/tabmenu';

interface PayloadMenuItem {
  label: string;
  payload: string;
}

@Component({
  selector: 'app-gallery',
  standalone: true,
  imports: [
    NgIf,
    NgForOf,
    CascadeSelectModule,
    PanelModule,
    NgClass,
    FileUploadModule,
    ButtonGroupModule,
    TabMenuModule
  ],
  templateUrl: './gallery.component.html',
  styleUrls: ['./gallery.component.css']
})
export class GalleryComponent implements OnInit {

  gallery: [ImageType] = [{id: '', likes: 0, date: '', liked: false}];

  selectedFile: File | null = null;

  previewUrl: string | ArrayBuffer | null = null;

  @Input()
  weddingId: string = '';

  @Input()
  guestId: string = '';

  sorts: Array<PayloadMenuItem> = [
    {
      label: 'Najnowsze',
      payload: 'date'
    },
    {
      label: 'Popularne',
      payload: 'likes'
    }
  ];

  activeSort: PayloadMenuItem = this.sorts[0];


  constructor(private route: ActivatedRoute,
              private galleryService: GalleryService) {
  }

  ngOnInit() {
    this.loadGallery();
  }

  onDownloadImageAction(imageId: string): void {
    const url = `/api/wedding/${this.weddingId}/gallery/${imageId}`;
    const link = document.createElement('a');
    link.href = url;
    link.download = `${imageId}.jpg`;
    link.style.display = 'none';

    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }

  onUploadAction(event: FileSelectEvent): void {
    console.log(event);
    this.selectedFile = event.currentFiles[0];
    this.previewUrl = URL.createObjectURL(this.selectedFile);
    const container = document.getElementById('show-selected-image-container');
    if (container) {
      container.style.display = 'flex';
    }
  }

  onSendButtonAction(): void {
    if (this.selectedFile) {
      this.galleryService.postImage(this.selectedFile, this.weddingId, this.guestId).subscribe({
        next: () => {
          this.onImageClearAction();
          this.loadGallery();
        },
        error: (err) => console.error('Error uploading image:', err)
      });
    }
  }

  onImageClearAction(): void {
    const container = document.getElementById('show-selected-image-container');
    if (container) {
      container.style.display = 'none';
    }
    this.selectedFile = null;
    this.previewUrl = null;
  }

  onSortTypeChanged(sort: MenuItem): void {
    this.activeSort = sort as PayloadMenuItem;
    this.loadGallery();
  }

  onLikeButtonAction(image: ImageType): void {
    image.liked = !image.liked;

    if (image.liked) {
      image.likes = image.likes + 1;
    }
    else {
      image.likes = image.likes - 1;
    }

    this.galleryService.changeImageLikes(this.weddingId, this.guestId, image.id, image.liked).subscribe({
      error: (error) => console.error('Error updating like:', error)
    });
  }

  loadGallery(): void {
    this.galleryService
      .getGalleryByWeddingId(this.weddingId, this.guestId, this.activeSort.payload)
      .subscribe({
        next: (weddingGallery) => this.gallery = weddingGallery,
        error: (error) => console.error('Error fetching gallery:', error)
      });
  }

}
