import {Component, OnInit, ViewChild} from '@angular/core';
import {catchError} from 'rxjs';
import {ActivatedRoute, Router} from '@angular/router';
import {GuestsService} from '../../services/guests.service';
import {GuestType} from '../../models/guests.type';
import {Button} from 'primeng/button';
import {Table, TableModule} from 'primeng/table';
import {PaginatorModule} from 'primeng/paginator';
import {CommonModule, NgClass} from '@angular/common';
import {TagModule} from 'primeng/tag';
import {InputTextModule} from 'primeng/inputtext';
import { InputIconModule } from 'primeng/inputicon';
import { IconFieldModule } from 'primeng/iconfield';
import {CardModule} from 'primeng/card';
import {DividerModule} from 'primeng/divider';
import {ConfirmationService, LazyLoadEvent, MessageService, SortEvent} from 'primeng/api';
import {FilterPipe} from '../../filter.pipe';
import {ToolbarModule} from 'primeng/toolbar';
import { ConfirmDialogModule } from 'primeng/confirmdialog';
import {MessagesModule} from 'primeng/messages';
import {DialogModule} from 'primeng/dialog';
import {CheckboxModule} from 'primeng/checkbox';
import {RadioButtonModule} from 'primeng/radiobutton';
import { v4 as uuidv4 } from 'uuid';
import {AutoCompleteModule} from 'primeng/autocomplete';

@Component({
  selector: 'app-guests',
  standalone: true,
  imports: [
    Button,
    TableModule,
    PaginatorModule,
    TagModule,
    InputTextModule,
    IconFieldModule,
    InputIconModule,
    CardModule,
    DividerModule,
    FilterPipe,
    CommonModule,
    ToolbarModule,
    ConfirmDialogModule,
    MessagesModule,
    DialogModule,
    CheckboxModule,
    RadioButtonModule,
    AutoCompleteModule,
  ],
  providers: [MessageService, ConfirmationService],
  templateUrl: './guests.component.html',
  styleUrl: './guests.component.css',
  host: { ngSkipHydration: 'true' },
})
export class GuestsComponent implements OnInit {

  @ViewChild('dt') dt!: Table;

  weddingGuests: GuestType[] = [];
  initialOrderOfGuests: GuestType[] = [];

  filteredGuestsNames: any[] = [];
  filteredGuestsSurnames: string[] = [];

  weddingId: string = '';
  guestId:string = '';

  searchText:any;
  isSorted:boolean |null=null;

  guestDialog: boolean = false;
  submitted: boolean = false;

  ageGuestOptions = [
    { label: 'Dorosły', value: 'Dorosły' },
    { label: 'Dziecko', value: 'Dziecko' },
    { label: 'Młodzież', value: 'Młodzież' },
    { label: 'Nie wiadomo', value: 'Brak' }
  ];

  guestForm!: GuestType;

  constructor(private router: Router,private route: ActivatedRoute,private GuestService: GuestsService,private confirmationService: ConfirmationService,private messageService: MessageService,) {}


  ngOnInit(): void {
    this.weddingId = this.route.snapshot.paramMap.get('weddingId') || '';
    this.guestId = this.route.snapshot.paramMap.get('guestId') || '';
    if (this.weddingId) {
      this.getGuests();
    } else {
      console.error('weddingId error');
    }
  }

  getGuests() {
    this.GuestService.getGuestsByWeddingId(this.weddingId, this.guestId).pipe(
      catchError((error) => {
        console.error('Error fetching quiz state:', error);
        throw error;
      })
    ).subscribe((weddingGuests) => {
      this.weddingGuests = weddingGuests;
      this.initialOrderOfGuests = [...weddingGuests];
    });

  }

  addGuest() {
    this.guestForm = {guestAge: 'Brak', guestName:'',guestSurname:'',id:''};
    this.submitted = false;
    this.guestDialog = true;
  }

  editGuest(guest: GuestType) {
    this.guestForm = { ...guest };
    this.guestDialog = true;
  }

  deleteGuest(guest: GuestType) {
    this.confirmationService.confirm({
      message: 'Czy na pewno chcesz usunąć tego gościa?',
      header: 'Potwierdzenie',
      icon: 'pi pi-exclamation-triangle',
      acceptLabel: 'Tak, usuń',
      rejectLabel: 'Anuluj',
      accept: () => {
        this.GuestService.sendGuestToDeleteByWeddingId(this.weddingId,this.guestId,guest).subscribe({
          next: () => {
            this.getGuests();
            this.messageService.add({
              severity: 'success',
              summary: 'Udało się!',
              detail: 'Gość usunięty',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Błąd!',
              detail: 'Nie udało się usunąć gościa',
              life: 3000
            });
          }
        });
      }
    });
  }

  getAdultGuestsCount(): number {
    return this.weddingGuests.filter(guest => guest.guestAge==="Dorosły").length;
  }

  getTeenGuestsCount(): number {
    return this.weddingGuests.filter(guest => guest.guestAge==="Młodzież").length;
  }

  getChildGuestsCount(): number {
    return this.weddingGuests.filter(guest => guest.guestAge==="Dziecko").length;
  }


  saveGuest() {
    this.submitted = true;

    if (this.guestForm.guestName?.trim()) {
      if (this.guestForm.id) {
        this.GuestService.sendGuestToEditByWeddingId(this.weddingId, this.guestId, this.guestForm).subscribe({
          next: () => {
            this.getGuests();
            this.messageService.add({
              severity: 'success',
              summary: 'Successful',
              detail: 'Guest Updated',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Error',
              detail: 'Failed to update guest',
              life: 3000
            });
          }
        });
      } else {
        this.guestForm.id = uuidv4();
        this.GuestService.sendGuestToCreateByWeddingId(this.weddingId, this.guestId, this.guestForm).subscribe({
          next: () => {
            this.getGuests();
            this.messageService.add({
              severity: 'success',
              summary: 'Successful',
              detail: 'Guest Created',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Error',
              detail: 'Failed to create guest',
              life: 3000
            });
          }
        });
      }

      this.weddingGuests = [...this.weddingGuests];
      this.guestDialog = false;
      this.guestForm = { guestAge: 'Brak', guestName: '', guestSurname: '', id: '' };
    }
  }



  sortTableData(event:any) {
    event.data.sort((data1:any, data2:any) => {
      let value1 = data1[event.field];
      let value2 = data2[event.field];
      let result = null;
      if (value1 == null && value2 != null) result = -1;
      else if (value1 != null && value2 == null) result = 1;
      else if (value1 == null && value2 == null) result = 0;
      else if (typeof value1 === 'string' && typeof value2 === 'string') result = value1.localeCompare(value2);
      else result = value1 < value2 ? -1 : value1 > value2 ? 1 : 0;

      return event.order * result;
    });
  }

  customSort(event: SortEvent) {
    if (this.isSorted == null) {
      this.isSorted = true;
      this.sortTableData(event);
    } else if (this.isSorted) {
      this.isSorted = false;
      this.sortTableData(event);
    } else if (!this.isSorted) {
      this.isSorted = null;
      this.weddingGuests = [...this.initialOrderOfGuests];
      this.dt.reset();
    }
  }

  getSeverity(status: string) {
    switch (status) {
      case "Dorosły":
        return 'success';
      case "Młodzież":
        return 'warning';
      case "Dziecko":
        return 'danger';
      default:
        return 'info';
    }
  }

  hideDialog() {
    this.guestDialog = false;
    this.submitted = false;
  }

  filterGuestsNames(event: any) {
    const query = event.query.toLowerCase();
    this.filteredGuestsNames = Array.from(
      new Set(
        this.weddingGuests
          .filter((guest) =>
            guest.guestName.toLowerCase().includes(query)
          )
          .map((guest: GuestType) => guest.guestName)
      )
    );
  }

  filterGuestsSurnames(event: any) {
    const query = event.query.toLowerCase();
    this.filteredGuestsSurnames = Array.from(
      new Set(
        this.weddingGuests
          .filter((guest) =>
            guest.guestSurname.toLowerCase().includes(query)
          )
          .map((guest: GuestType) => guest.guestSurname)
      )
    );
  }

}
